@@ -1,18 +1,19 @@
 """
 Trading project:
-- testing
-- trading
+    - testing
+    - trading
 
 """
 
-#!/usr/bin/python
+# !/usr/bin/python
 # -*- coding: utf-8 -*-
 # used ta by Darío López Padial (Bukosabino https://github.com/bukosabino/ta)
 
 
 # TODO:
 #   add inner class with non-trading utils
-#   all talib patterns
+#   debug neural networks
+#   do quick-trade-tuner better
 
 import itertools
 import random
@@ -22,13 +23,14 @@ from typing import Dict, List, Tuple
 
 import numpy as np
 import pandas as pd
+import quick_trade.brokers as brokers
 import ta
 import ta.momentum
 import ta.others
 import ta.trend
 import ta.volatility
 import ta.volume
-from binance.client import Client
+import talib
 from plotly.graph_objs import Line
 from plotly.subplots import make_subplots
 from pykalman import KalmanFilter
@@ -37,114 +39,6 @@ from scipy import signal
 from sklearn.preprocessing import MinMaxScaler
 from tensorflow.keras.layers import Dropout, Dense, LSTM
 from tensorflow.keras.models import Sequential, load_model
-import datetime
-import talib
-
-
-class BinanceTradingClient(Client):
-    ordered: bool = False
-    __side__: str
-    quantity: float
-    ticker: str
-    order: Dict[str, typing.Any]
-
-    def order_create(self,
-                     side: str,
-                     ticker: str = 'None',
-                     quantity: float = 0.0,
-                     credit_leverage: float = 1.0,
-                     *args,
-                     **kwargs):
-        self.__side__ = side
-        self.ticker = ticker
-        if '_moneys_' in kwargs:
-            if quantity > kwargs['_moneys_']:
-                quantity -= utils.min_admit(kwargs['rounding_bet'])
-            quantity = round(quantity, kwargs['rounding_bet'])
-            if quantity > kwargs['_moneys_'] or (quantity % utils.min_admit(kwargs['rounding_bet']) != 0):
-                utils.logger.error(f'invalid quantity: {quantity}, moneys: {kwargs["_moneys_"]}')
-        if side == 'Buy':
-            self.order = self.order_market_buy(symbol=ticker, quantity=quantity)
-        elif side == 'Sell':
-            self.order = self.order_market_sell(symbol=ticker, quantity=quantity)
-        self.order_id = self.order['orderId']
-        self.quantity = quantity
-        self.ordered = True
-
-    def get_ticker_price(self,
-                         ticker: str) -> float:
-        return float(self.get_symbol_ticker(symbol=ticker)['price'])
-
-    @staticmethod
-    def get_data(ticker: str = 'None',
-                 interval: str = 'None',
-                 **get_kw) -> pd.DataFrame:
-        return utils.get_binance_data(ticker, interval, **get_kw)
-
-    def new_order_buy(self,
-                      ticker: str = 'None',
-                      quantity: float = 0.0,
-                      credit_leverage: float = 1.0,
-                      *args,
-                      **kwargs):
-        self.order_create('Buy',
-                          ticker=ticker,
-                          quantity=quantity,
-                          credit_leverage=credit_leverage,
-                          *args,
-                          **kwargs)
-        utils.logger.info('client buy')
-
-    def new_order_sell(self,
-                       ticker: str = 'None',
-                       quantity: float = 0.0,
-                       credit_leverage: float = 1.0,
-                       *args,
-                       **kwargs):
-        self.order_create('Sell',
-                          ticker=ticker,
-                          quantity=quantity,
-                          credit_leverage=credit_leverage,
-                          *args,
-                          **kwargs)
-        utils.logger.info('client sell')
-
-    def get_data_historical(self,
-                            ticker: str = 'None',
-                            start: str = '15 Dec 2020',
-                            interval: str = '1m',
-                            limit: int = 1000,
-                            start_type: str = '%d %b %Y'):
-        start_date = datetime.datetime.strptime(start, start_type)
-        today = datetime.datetime.now()
-
-        klines = self.get_historical_klines(ticker,
-                                            interval,
-                                            start_date.strftime("%d %b %Y %H:%M:%S"),
-                                            today.strftime("%d %b %Y %H:%M:%S"),
-                                            limit)
-        data = pd.DataFrame(klines,
-                            columns=['timestamp', 'Open', 'High', 'Low', 'Close', 'Volume', 'close_time', 'quote_av',
-                                     'trades', 'tb_base_av', 'tb_quote_av', 'ignore'])
-        data['timestamp'] = pd.to_datetime(data['timestamp'], unit='ms')
-
-        data.set_index('timestamp', inplace=True)
-        return data.astype(float)
-
-    def exit_last_order(self):
-        if self.ordered:
-            if self.__side__ == 'Sell':
-                self.new_order_buy(self.ticker, self.quantity)
-            elif self.__side__ == 'Buy':
-                self.new_order_sell(self.ticker, self.quantity)
-            self.__side__ = 'Exit'
-            self.ordered = False
-        utils.logger.info('client exit')
-
-    def get_balance_ticker(self, ticker: str) -> float:
-        for asset in self.get_account()['balances']:
-            if asset['asset'] == ticker:
-                return float(asset['free'])
 
 
 class Trader(object):
@@ -154,11 +48,11 @@ class Trader(object):
     df:       |   dataframe  |  data of chart
     interval: |     str      |  interval of df.
     one of:
-    1m    30m    3h    3M
-    2m    45m    4h    6M
-    3m    1h     1d
-    5m    90m    1w
-    15m   2h     1M
+    1m    30m    3h    1M
+    2m    45m    4h    3M
+    3m    1h     1d    6M
+    5m    90m    3d
+    15m   2h     1w
 
     """
     profit_calculate_coef: float
@@ -191,10 +85,11 @@ class Trader(object):
     backtest_out: pd.DataFrame
     open_lot_prices: List[float]
     realtime_returns: Dict[str, Dict[str, typing.Union[str, float]]]
-    client: BinanceTradingClient
+    client: brokers.TradingClient
     __last_stop_loss: float
     __last_take_profit: float
     model: Sequential
+    returns_strategy: List[float]
 
     def __init__(self,
                  ticker: str = 'AAPL',
@@ -234,6 +129,8 @@ class Trader(object):
             self.profit_calculate_coef = 1 / (6 * 365)
         elif interval == '1d':
             self.profit_calculate_coef = 1 / 365
+        elif interval == '3d':
+            self.profit_calculate_coef = 1 / (365 / 3)
         elif interval == '1w':
             self.profit_calculate_coef = 1 / 52
         elif interval == '1M':
@@ -329,6 +226,7 @@ class Trader(object):
         for i in range(length):
             return_list.append(start + mean_diff * i)
         self.mean_diff = mean_diff
+        utils.logger.debug(f'in linear: self.mean_diff={mean_diff}')
         return np.array(return_list)
 
     def __get_stop_take(self, sig: utils.PREDICT_TYPE) -> Dict[str, float]:
@@ -362,6 +260,8 @@ class Trader(object):
                 take = self.open_price
             if self.stop_loss is not np.inf:
                 _stop_loss = self.open_price
+        utils.logger.debug(
+            f'stop loss: {_stop_loss} ({self.stop_loss} pips), take profit: {take} ({self.take_profit} pips)')
 
         return {'stop': _stop_loss,
                 'take': take}
@@ -371,10 +271,12 @@ class Trader(object):
         frame_to_diff:  |   pd.Series  |  example:  Trader.df['Close']
         """
         self.returns = list(np.digitize(frame_to_diff.diff(), bins=[0]))
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_buy_hold(self, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
         self.returns = [utils.BUY for _ in range(len(self.df))]
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_2_sma(self,
@@ -391,12 +293,12 @@ class Trader(object):
                 Line(
                     name=f'SMA{fast}',
                     y=SMA1.values,
-                    line=dict(width=utils.SUB_LINES_WIDTH, color=utils.G)), 1, 1)
+                    line=dict(width=utils.SUB_LINES_WIDTH, color=utils.GREEN)), 1, 1)
             self.fig.add_trace(
                 Line(
                     name=f'SMA{slow}',
                     y=SMA2.values,
-                    line=dict(width=utils.SUB_LINES_WIDTH, color=utils.R)), 1, 1)
+                    line=dict(width=utils.SUB_LINES_WIDTH, color=utils.RED)), 1, 1)
 
         for SMA13, SMA26 in zip(SMA1, SMA2):
             if SMA26 < SMA13:
@@ -405,6 +307,7 @@ class Trader(object):
                 self.returns.append(utils.SELL)
             else:
                 self.returns.append(utils.EXIT)
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_3_sma(self,
@@ -421,7 +324,7 @@ class Trader(object):
 
         if plot:
             for SMA, Co, name in zip([SMA1, SMA2, SMA3],
-                                     [utils.G, utils.B, utils.R],
+                                     [utils.GREEN, utils.BLUE, utils.RED],
                                      [fast, mid, slow]):
                 self.fig.add_trace(
                     Line(
@@ -437,6 +340,7 @@ class Trader(object):
             else:
                 self.returns.append(utils.EXIT)
 
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_3_ema(self,
@@ -453,7 +357,7 @@ class Trader(object):
 
         if plot:
             for ema, Co, name in zip([ema3.values, ema21.values, ema46.values],
-                                     [utils.G, utils.B, utils.R], [slow, mid, fast]):
+                                     [utils.GREEN, utils.BLUE, utils.RED], [slow, mid, fast]):
                 self.fig.add_trace(
                     Line(
                         name=f'SMA{name}',
@@ -467,6 +371,7 @@ class Trader(object):
                 self.returns.append(utils.SELL)
             else:
                 self.returns.append(utils.EXIT)
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_macd(self,
@@ -484,6 +389,7 @@ class Trader(object):
                 self.returns.append(utils.SELL)
             else:
                 self.returns.append(utils.EXIT)
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_exp_diff(self,
@@ -500,6 +406,7 @@ class Trader(object):
                     y=exp.values.T[0],
                     line=dict(width=utils.SUB_LINES_WIDTH)), 1, 1)
 
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_rsi(self,
@@ -510,13 +417,13 @@ class Trader(object):
                      *args,
                      **rsi_kwargs) -> utils.PREDICT_TYPE_LIST:
         self.returns = []
-        rsi = ta.momentum.rsi(self.df['Close'], **rsi_kwargs)
+        rsi = ta.momentum.rsi(close=self.df['Close'], **rsi_kwargs)
         flag: utils.PREDICT_TYPE = utils.EXIT
 
         for val in rsi.values:
-            if val < minimum and val is not pd.NA:
+            if val < minimum:
                 flag = utils.BUY
-            elif val > maximum and val is not pd.NA:
+            elif val > maximum:
                 flag = utils.SELL
             elif flag == utils.BUY and val < max_mid:
                 flag = utils.EXIT
@@ -524,6 +431,7 @@ class Trader(object):
                 flag = utils.EXIT
             self.returns.append(flag)
 
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_parabolic_SAR(self, plot: bool = True, *args, **sar_kwargs) -> utils.PREDICT_TYPE_LIST:
@@ -562,10 +470,17 @@ class Trader(object):
         signal_ = _MACD_.macd_signal()
         macd_ = _MACD_.macd()
         histogram: pd.DataFrame = pd.DataFrame(macd_.values - signal_.values)
-        self.returns = utils.digit(histogram.diff().values)
+        for element in histogram.diff().values:
+            if element == 0:
+                self.returns.append(utils.EXIT)
+            elif element > 0:
+                self.returns.append(utils.BUY)
+            else:
+                self.returns.append(utils.SELL)
+        self.set_open_stop_and_take()
         return self.returns
 
-    def strategy_regression_model(self, plot: bool = True, *args, **kwargs):
+    def strategy_regression_model(self, plot: bool = True, *args, **kwargs):  # TODO: fix
         self.returns = [utils.EXIT for i in range(self._regression_inputs - 1)]
         data_to_pred: np.ndarray = np.array(
             utils.get_window(np.array([self.df['Close'].values]).T, self._regression_inputs)
@@ -589,16 +504,17 @@ class Trader(object):
                 Line(
                     name='predict',
                     y=filt,
-                    line=dict(width=utils.SUB_LINES_WIDTH, color=utils.C)),
+                    line=dict(width=utils.SUB_LINES_WIDTH, color=utils.CYAN)),
                 row=1,
                 col=1)
+        self.set_open_stop_and_take()
         return self.returns, filt
 
     def get_network_regression(self,
                                dataframes: typing.Iterable[pd.DataFrame],
                                inputs: int = utils.REGRESSION_INPUTS,
                                network_save_path: str = './model_regression.h5',
-                               **fit_kwargs) -> Sequential:
+                               **fit_kwargs) -> Sequential:  # TODO: fix
         """based on
         https://medium.com/@randerson112358/stock-price-prediction-using-python-machine-learning-e82a039ac2bb
         """
@@ -631,7 +547,7 @@ class Trader(object):
 
     def prepare_scaler(self,
                        dataframe: pd.DataFrame,
-                       regression_net: bool = True) -> np.ndarray:
+                       regression_net: bool = True) -> np.ndarray:  # TODO: fix
         self.scaler = MinMaxScaler(feature_range=(0, 1))
         data: pd.DataFrame
         dataset: np.ndarray
@@ -651,7 +567,9 @@ class Trader(object):
                             loss: str = 'mse',
                             metrics: typing.Iterable[str] = ['mse'],
                             network_save_path: str = './model_predicting.h5',
-                            **fit_kwargs) -> Tuple[Sequential, Dict[str, List[float]], Tuple[np.ndarray, np.ndarray]]:
+                            **fit_kwargs) -> Tuple[Sequential,
+                                                   Dict[str, List[float]],
+                                                   Tuple[np.ndarray, np.ndarray]]:  # TODO: fix
         """
         getting trained neural network to trading.
         dataframes:  | typing.Iterable[pd.DataFrame] |   list of pandas dataframes with columns:
@@ -661,7 +579,7 @@ class Trader(object):
             'Close'
             'Volume'
         optimizer:    |        str         |   optimizer for .compile of network.
-        filter_:      |        str         |    filter to training.
+        filter_:      |        str         |    filter for training.
         filter_kwargs:|       dict         |    named arguments for the filter.
         loss:         |        str         |   loss for .compile of network.
         metrics:      |typing.Iterable[str]|   metrics for .compile of network:
@@ -712,14 +630,15 @@ class Trader(object):
         return self.model, self.history, self.training_set
 
     def strategy_random_pred(self, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
-        self.returns = [random.randint(0, 2) for i in range(len(self.df))]
+        self.returns = [random.choice([utils.EXIT, utils.SELL, utils.BUY]) for i in range(len(self.df))]
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_with_network(self,
                               rounding: int = 0,
                               _rounding_prediction_func=round,
                               *args,
-                              **kwargs) -> utils.PREDICT_TYPE_LIST:
+                              **kwargs) -> utils.PREDICT_TYPE_LIST:  # TODO: fix
         """
         :param rounding: rounding degree for _rounding_prediction_func
         :param _rounding_prediction_func: A function that will be used to round off the neural network result.
@@ -731,6 +650,7 @@ class Trader(object):
         for e, i in enumerate(preds):
             preds[e] = _rounding_prediction_func(i[0], rounding)
         self.returns = list(preds)
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_supertrend(self, plot: bool = True, *st_args, **st_kwargs) -> utils.PREDICT_TYPE_LIST:
@@ -742,14 +662,14 @@ class Trader(object):
         if plot:
             self.fig.add_trace(Line(y=st.get_supertrend_upper(),
                                     name='supertrend upper',
-                                    line=dict(width=utils.SUB_LINES_WIDTH, color=utils.R)))
+                                    line=dict(width=utils.SUB_LINES_WIDTH, color=utils.RED)))
             self.fig.add_trace(Line(y=st.get_supertrend_lower(),
                                     name='supertrend lower',
-                                    line=dict(width=utils.SUB_LINES_WIDTH, color=utils.G)))
+                                    line=dict(width=utils.SUB_LINES_WIDTH, color=utils.GREEN)))
         self.stop_losses = list(st.get_supertrend())
         self.returns = list(st.get_supertrend_strategy_returns())
-        self.set_open_stop_and_take(set_stop=False)
         self.stop_losses[0] = np.inf if self.returns[0] == utils.SELL else -np.inf
+        self.set_open_stop_and_take(set_stop=False)
         return self.returns
 
     def strategy_bollinger(self,
@@ -791,6 +711,7 @@ class Trader(object):
                 if flag == utils.BUY and close >= mid:
                     flag = utils.EXIT
             self.returns.append(flag)
+        self.set_open_stop_and_take()
         return self.returns
 
     def get_heikin_ashi(self, df: pd.DataFrame = pd.DataFrame()) -> pd.DataFrame:
@@ -864,7 +785,7 @@ class Trader(object):
 
             self.fig.add_trace(Line(y=senkou_span_a,
                                     fill=None,
-                                    line_color=utils.R,
+                                    line_color=utils.RED,
                                     ))
             self.fig.add_trace(Line(
                 y=senkou_span_b,
@@ -912,9 +833,10 @@ class Trader(object):
                     self.stop_losses.append(max_cloud + stop_loss_adder)
         self.set_open_stop_and_take(set_take=True,
                                     set_stop=False)
+        self.set_open_stop_and_take()
         return self.returns
 
-    def inverse_strategy(self, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
+    def inverse_strategy(self, swap_tpop_take: bool = True, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
         """
         makes signals inverse:
         buy = sell.
@@ -933,6 +855,8 @@ class Trader(object):
                 flag = utils.EXIT
             returns.append(flag)
         self.returns = returns
+        if swap_tpop_take:
+            self.stop_losses, self.take_profits = self.take_profits, self.stop_losses
         return self.returns
 
     def backtest(self,
@@ -1055,11 +979,13 @@ trades: {self.trades}
 profits: {self.profits}
 mean year percentage profit: {self.year_profit}%
 winrate: {self.winrate}%"""
+        utils.logger.info(f'trader info: {self.info}')
         if print_out:
             print(self.info)
+        self.returns_strategy = list(pd.Series(self.deposit_history).diff().values)
         self.backtest_out_no_drop = pd.DataFrame(
             (self.deposit_history, self.stop_losses, self.take_profits, self.returns,
-             self.open_lot_prices, data_column, self.linear, pd.Series(self.deposit_history).diff().values),
+             self.open_lot_prices, data_column, self.linear, self.returns_strategy),
             index=[
                 f'deposit ({column})', 'stop loss', 'take profit',
                 'predictions', 'open deal/lot', column,
@@ -1074,7 +1000,7 @@ winrate: {self.winrate}%"""
                     y=self.backtest_out_no_drop['returns'].values,
                     line=dict(color=utils.COLOR_DEPOSIT),
                     name='returns'
-                    ),
+                ),
                 row=3,
                 col=1
             )
@@ -1085,11 +1011,11 @@ winrate: {self.winrate}%"""
                 open=self.df['Open'],
                 row=1,
                 col=1,
-                name=self.ticker)
+                name=f'{self.ticker} {self.interval}')
             self.fig.add_trace(
                 Line(
                     y=self.take_profits,
-                    line=dict(width=utils.TAKE_STOP_OPN_WIDTH, color=utils.G),
+                    line=dict(width=utils.TAKE_STOP_OPN_WIDTH, color=utils.GREEN),
                     opacity=utils.STOP_TAKE_OPN_ALPHA,
                     name='take profit'),
                 row=1,
@@ -1097,7 +1023,7 @@ winrate: {self.winrate}%"""
             self.fig.add_trace(
                 Line(
                     y=self.stop_losses,
-                    line=dict(width=utils.TAKE_STOP_OPN_WIDTH, color=utils.R),
+                    line=dict(width=utils.TAKE_STOP_OPN_WIDTH, color=utils.RED),
                     opacity=utils.STOP_TAKE_OPN_ALPHA,
                     name='stop loss'),
                 row=1,
@@ -1105,7 +1031,7 @@ winrate: {self.winrate}%"""
             self.fig.add_trace(
                 Line(
                     y=self.open_lot_prices,
-                    line=dict(width=utils.TAKE_STOP_OPN_WIDTH, color=utils.B),
+                    line=dict(width=utils.TAKE_STOP_OPN_WIDTH, color=utils.BLUE),
                     opacity=utils.STOP_TAKE_OPN_ALPHA,
                     name='open lot'),
                 row=1,
@@ -1140,7 +1066,7 @@ winrate: {self.winrate}%"""
                     [preds['buyind'], preds['sellind'], preds['exitind']],
                     [preds['bprice'], preds['sprice'], preds['eprice']],
                     ['triangle-up', 'triangle-down', 'triangle-left'],
-                    [utils.G, utils.R, utils.B]
+                    [utils.GREEN, utils.RED, utils.BLUE]
             ):
                 self.fig.add_scatter(
                     mode='markers',
@@ -1162,13 +1088,13 @@ winrate: {self.winrate}%"""
                    height: int = 900,
                    width: int = 1300,
                    template: str = 'plotly_dark',
-                   row_heights: list = [100, 160, 70],
+                   row_heights: list = [10, 16, 7],
                    **subplot_kwargs):
         """
         :param height: window height
         :param width: window width
         :param template: plotly template
-        :param row_heights: standard [100, 160]
+        :param row_heights: standard
         """
         self.fig = make_subplots(3, 1, row_heights=row_heights, **subplot_kwargs)
         self.fig.update_layout(
@@ -1194,7 +1120,7 @@ winrate: {self.winrate}%"""
         """
         :param second_returns: returns of strategy
         :param first_returns: returns of strategy
-        :param mode:  mode of combining
+        :param mode:  mode of combining:
 
             example :
                 mode = 'minimalist':
@@ -1333,24 +1259,21 @@ winrate: {self.winrate}%"""
         # get prediction
         predict = self.returns[-1]
         predict = utils.convert_signal_str(predict)
-        if self.__exit_order__:
+        if self.__exit_order__ and self._old_predict == predict:
             predict = 'Exit'
+        if predict != 'Exit':
+            self.__exit_order__ = False
 
         # trading
         self.__last_stop_loss = self.stop_losses[-1]
         self.__last_take_profit = self.take_profits[-1]
         if self._old_predict != predict:
             utils.logger.info(f'open lot {predict}')
-            for sig, close_ in zip(self.returns[::-1],
-                                   close[::-1]):
-                if sig != utils.EXIT:
-                    self.open_price = close_
-                    break
+            self.open_price = close[-1]
             if trading_on_client:
 
                 if predict == 'Exit':
                     self.client.exit_last_order()
-                    self.__exit_order__ = True
 
                 else:
                     _moneys_ = self.client.get_balance_ticker(second_symbol_of_ticker)
@@ -1358,13 +1281,11 @@ winrate: {self.winrate}%"""
                     if coin_lotsize_division:
                         _moneys_ /= ticker_price
                     if bet_for_trading_on_client is not np.inf:
-                        bet = bet_for_trading_on_client
+                        bet = bet_for_trading_on_client / ticker_price
                     else:
                         bet = _moneys_
                     if bet > _moneys_:
                         bet = _moneys_
-                    if coin_lotsize_division:
-                        bet /= ticker_price
                     self.client.exit_last_order()
 
                     self.client.order_create(predict,
@@ -1384,7 +1305,7 @@ winrate: {self.winrate}%"""
 
     def realtime_trading(self,
                          strategy,
-                         ticker: str = utils.BASE_TICKER,
+                         ticker: str = 'AAPL',
                          get_data_kwargs: Dict[str, typing.Any] = {},
                          sleeping_time: float = 60.0,
                          print_out: bool = True,
@@ -1410,7 +1331,7 @@ winrate: {self.winrate}%"""
         :param strategy_args: arguments to -strategy.
         """
 
-        self.realtie_returns = {}
+        self.realtime_returns = {}
         self.ticker = ticker
         try:
             __now__ = time.time()
@@ -1426,48 +1347,57 @@ winrate: {self.winrate}%"""
                     coin_lotsize_division=coin_lotsize_division)
 
                 index = f'{self.ticker}, {time.ctime()}'
+                utils.logger.info(f"trading prediction at {index}: {prediction}")
                 if print_out:
                     print(index, prediction)
-                utils.logger.info(f"trading prediction at {index}: {prediction}")
-                self.realtie_returns[index] = prediction
+                self.realtime_returns[index] = prediction
                 while True:
                     if not self.__exit_order__:
                         price = self.client.get_ticker_price(ticker)
                         min_ = min(self.__last_stop_loss, self.__last_take_profit)
                         max_ = max(self.__last_stop_loss, self.__last_take_profit)
-                        if (not min_ < price < max_) and self._old_predict != utils.EXIT:
+                        if (not (min_ < price < max_)) and prediction["predict"] != 'Exit':
                             self.__exit_order__ = True
                             utils.logger.info('exit lot')
                             prediction['predict'] = 'Exit'
                             prediction['currency close'] = price
                             index = f'{self.ticker}, {time.ctime()}'
                             utils.logger.info(f"trading prediction exit in sleeping at {index}: {prediction}")
-                            self.realtie_returns[index] = prediction
+                            if print_out:
+                                print(f"trading prediction exit in sleeping at {index}: {prediction}")
+                            self.realtime_returns[index] = prediction
                             if trading_on_client:
                                 self.client.exit_last_order()
                     if not (time.time() < (__now__ + sleeping_time)):
-                        self._old_predict = prediction['predict']
-                        self.__exit_order__ = False
+                        self._old_predict = utils.convert_signal_str(self.returns[-1])
                         __now__ += sleeping_time
                         break
 
         except Exception as e:
+            utils.logger.critical('error :(', exc_info=True)
             raise e
 
     def log_data(self, *args, **kwargs):
         self.fig.update_yaxes(row=1, col=1, type='log')
+        utils.logger.info('trader log data')
 
     def log_deposit(self, *args, **kwargs):
         self.fig.update_yaxes(row=2, col=1, type='log')
+        utils.logger.info('trader log deposit')
+
+    def log_returns(self, *args, **kwargs):
+        self.fig.update_yaxes(row=3, col=1, type='log')
+        utils.logger.info('trader log returns')
 
     def load_model(self, path: str, *args, **kwargs):
         self.model = load_model(path)
 
-    def set_client(self, your_client: BinanceTradingClient, *args, **kwargs):
+    def set_client(self, your_client: brokers.TradingClient, *args, **kwargs):
         """
         :param your_client: trading client
         """
         self.client = your_client
+        utils.logger.info('trader set client')
 
     def convert_signal(self,
                        old: utils.PREDICT_TYPE = utils.SELL,
@@ -1479,6 +1409,7 @@ winrate: {self.winrate}%"""
         for pos, val in enumerate(self.returns):
             if val == old:
                 self.returns[pos] = new
+        utils.logger.debug(f'trader signals converted: {old} >> {new}')
         return self.returns
 
     def set_open_stop_and_take(self,
@@ -1522,13 +1453,15 @@ winrate: {self.winrate}%"""
                 self.take_profits.append(take_flag)
             if set_stop:
                 self.stop_losses.append(stop_flag)
+        utils.logger.debug(f'trader stop loss: {stop_loss}, trader take profit: {take_profit}')
 
-    def set_credit_leverages(self, credit_lev: float = 0.0, *args, **kwargs):
+    def set_credit_leverages(self, credit_lev: float = 1.0, *args, **kwargs):
         """
         Sets the leverage for bets.
         :param credit_lev: leverage in points
         """
         self.credit_leverages = [credit_lev for i in range(len(self.df['Close']))]
+        utils.logger.debug(f'trader credit leverage: {credit_lev}')
 
     def _window_(self,
                  column: str,
@@ -1567,6 +1500,7 @@ winrate: {self.winrate}%"""
                 self.returns.append(flag)
             else:
                 self.returns.append(flag)
+        self.set_open_stop_and_take()
         return self.returns
 
     def find_DBLHC_DBHLC(self, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
@@ -1609,12 +1543,14 @@ winrate: {self.winrate}%"""
         for e, (high, low, open_, close) in enumerate(
                 zip(
                     self._window_('High'), self._window_('Low'),
-                    self._window_('Open'), self._window_('Close')), 1):
+                    self._window_('Open'), self._window_('Close')
+                ), 1):
             if high[0] == high[1]:
                 flag = utils.BUY
             elif low[0] == low[1]:
                 flag = utils.SELL
             self.returns.append(flag)
+        self.set_open_stop_and_take()
         return self.returns
 
     def find_PPR(self, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
@@ -1634,6 +1570,7 @@ winrate: {self.winrate}%"""
                      ) == high[1] and close[2] < close[1] and low[2] > low[0]:
                 flag = utils.SELL
             self.returns.append(flag)
+        self.set_open_stop_and_take()
         return self.returns
 
     def is_doji(self, *args, **kwargs) -> List[bool]:
@@ -1647,9 +1584,10 @@ winrate: {self.winrate}%"""
                 ret.append(True)
             else:
                 ret.append(False)
+        self.set_open_stop_and_take()
         return ret
 
-    def find_all_talib_paterns(self, *args, **kwargs):
+    def find_all_talib_patterns(self, *args, **kwargs):
         open_ = self.df['Open']
         high = self.df['High']
         low = self.df['Low']
@@ -1720,4 +1658,5 @@ winrate: {self.winrate}%"""
         ])
 
         patterns = map(utils.anti_set_, patterns)
+        self.set_open_stop_and_take()
         return self.multi_strategy_collider(*patterns, mode='super')
