@@ -1,5 +1,4 @@
 import datetime as dt
-import itertools
 import json
 import logging
 import os
@@ -16,10 +15,10 @@ PREDICT_TYPE_LIST: type = List[PREDICT_TYPE]
 SETED_TYPE: type = Union[PREDICT_TYPE, float]
 SETED_TYPE_LIST: type = List[SETED_TYPE]
 
-R: str = '#ff0000'
-G: str = '#55ff00'
-B: str = '#0015ff'
-C: str = 'cyan'
+RED: str = '#ff0000'
+GREEN: str = '#55ff00'
+BLUE: str = '#0015ff'
+CYAN: str = 'cyan'
 REGRESSION_INPUTS: int = 100
 BUY: PREDICT_TYPE = 1
 SELL: PREDICT_TYPE = 0
@@ -33,14 +32,15 @@ __author__: str = 'Vlad Kochetov'
 __credits__: List[str] = ["Hemerson Tacon -- Stack overflow",
                           "hpaulj -- Stack overflow",
                           "furas -- Stack overflow",
+                          "Devin Jeanpierre (edit: wjandrea) -- Stack overflow",
                           "Войтенко Николай Поликарпович (Vojtenko Nikolaj Polikarpovich) -- helped me test the "
                           "system of interaction with the binance crypto exchange with 50 dollars.",
+
                           "https://fxgears.com/index.php?threads/how-to-acquire-free-historical-tick-and-bar-data-for"
                           "-algo-trading-and-backtesting-in-2020-stocks-forex-and-crypto-currency.1229/#post-19305"
                           " -- binance get historical data method"]
-__version__: str = "3.9.5"
+__version__: str = "4.0"
 
-BASE_TICKER: str = '^DJI'
 SCATTER_SIZE: float = 12.0
 SCATTER_ALPHA: float = 1.0
 TAKE_STOP_OPN_WIDTH: float = 1.0
@@ -54,10 +54,9 @@ COLOR_DEPOSIT: str = 'white'
 DEPO_COLOR_UP: str = 'green'
 DEPO_COLOR_DOWN: str = 'red'
 
-FILE_LOG_NAME: str = 'trading.log'
 logger = logging.getLogger()
 logger.setLevel(30)
-logging.basicConfig(level=20, filename=FILE_LOG_NAME)
+logging.basicConfig(level=20, filename='trading.log', format='%(levelname)s::%(name)s::%(asctime)s::    %(message)s')
 
 
 class SuperTrendIndicator(object):
@@ -147,16 +146,6 @@ class SuperTrendIndicator(object):
         return df
 
 
-def expansion_with_shear(values, ins=EXIT) -> PREDICT_TYPE_LIST:
-    ret: PREDICT_TYPE_LIST = []
-    for value in values:
-        for column in range(4):
-            ret.append(value)
-    for i in range(3):
-        ret.insert(0, ins)
-    return ret[:-3]
-
-
 def set_(data: Any) -> SETED_TYPE_LIST:
     ret: List[Any] = list(data.copy())
     e: int
@@ -166,31 +155,6 @@ def set_(data: Any) -> SETED_TYPE_LIST:
     return ret
 
 
-def to_4_col_df(data: Any, *columns) -> pd.DataFrame:
-    """
-    data:  |  array-like  |  data to converting
-
-    returns:
-    pd.DataFrame with 4 your columns
-
-    """
-    predict: List[List[float]] = [[] for _ in range(4)]
-    for it in range(4):
-        for i in range(it, len(data), 4):
-            predict[it].append(data[i])
-    return pd.DataFrame(predict, index=columns).T
-
-
-def inverse_4_col_df(df: pd.DataFrame, columns: List[Any]) -> pd.DataFrame:
-    df: pd.DataFrame = pd.DataFrame(df)
-    ret: np.ndarray = df.values
-    rets: List[pd.DataFrame] = []
-    for column in columns:
-        rets.append(
-            pd.DataFrame(itertools.chain.from_iterable(ret), columns=[column]))
-    return pd.concat(rets, axis=1)
-
-
 def get_IEX_minutely(ticker: str, undo_days: int) -> pd.DataFrame:
     tuples: List[Tuple[int]] = []
     returns: pd.DataFrame
@@ -233,18 +197,6 @@ def anti_set_(seted: List[Any], _nan_num: float = 18699.9) -> List[Any]:
     return ret
 
 
-def digit(data) -> PREDICT_TYPE_LIST:
-    ret: PREDICT_TYPE_LIST = []
-    for element in list(data):
-        if element == 0:
-            ret.append(EXIT)
-        elif element > 0:
-            ret.append(BUY)
-        else:
-            ret.append(SELL)
-    return ret
-
-
 def get_window(values, window_length: int) -> List[Any]:
     ret: List[Any] = []
     for e, i in enumerate(values[:len(values) - window_length + 1]):
@@ -252,8 +204,8 @@ def get_window(values, window_length: int) -> List[Any]:
     return ret
 
 
-def get_binance_data(ticker: str = "BNBBTC", interval: str = "1m", date_index: bool = False):
-    url: str = f"https://api.binance.com/api/v1/klines?symbol={ticker}&interval={interval}"
+def get_binance_data(ticker: str = "BNBBTC", interval: str = "1m", date_index: bool = False, limit=500):
+    url: str = f"https://api.binance.com/api/v1/klines?symbol={ticker}&interval={interval}&limit={limit}"
     data: List[List[Any]] = json.loads(requests.get(url).text)
     df: pd.DataFrame = pd.DataFrame(data)
     df.columns = ["open_time",
@@ -268,10 +220,6 @@ def get_binance_data(ticker: str = "BNBBTC", interval: str = "1m", date_index: b
 
 
 def min_admit(r: int) -> float:
-    """
-
-    to math.floor analogue.
-    """
     return round(float('0.' + '0' * (r - 1) + '1'), r)
 
 
@@ -283,6 +231,7 @@ def convert_signal_str(predict: PREDICT_TYPE) -> str:
     elif predict == EXIT:
         return 'Exit'
 
+
 def ta_lib_to_returns(talib_returns: pd.Series, exit_=EXIT, *args, **kwargs) -> PREDICT_TYPE_LIST:
     return list(talib_returns.replace({-200: SELL,
                                        200: BUY,
@@ -291,5 +240,5 @@ def ta_lib_to_returns(talib_returns: pd.Series, exit_=EXIT, *args, **kwargs) ->
                                        0: exit_}).values)
 
 
-def ta_lib_collider_all(data, *args, **kwargs):
+def ta_lib_collider_all(data: pd.Series, *args, **kwargs) -> PREDICT_TYPE_LIST:
     return ta_lib_to_returns(data, exit_=np.nan)
